@@ -1,30 +1,32 @@
 #!/usr/bin/env python
 # encoding: utf-8
 #
-#    Copyright (c) 2013-2014+ Anton Tyurin <noxiouz@yandex.ru>
-#    Copyright (c) 2013-2014 Other contributors as noted in the AUTHORS file.
+#   Copyright (c) 2013-2014+ Anton Tyurin <noxiouz@yandex.ru>
+#   Copyright (c) 2013-2014 Other contributors as noted in the AUTHORS file.
 #
-#    This file is part of Cocaine.
+#   This file is part of Cocaine.
 #
-#    Cocaine is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU Lesser General Public License as published by
-#    the Free Software Foundation; either version 3 of the License, or
-#    (at your option) any later version.
+#   Cocaine is free software; you can redistribute it and/or modify
+#   it under the terms of the GNU Lesser General Public License as published by
+#   the Free Software Foundation; either version 3 of the License, or
+#   (at your option) any later version.
 #
-#    Cocaine is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#    GNU Lesser General Public License for more details.
+#   Cocaine is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+#   GNU Lesser General Public License for more details.
 #
-#    You should have received a copy of the GNU Lesser General Public License
-#    along with this program. If not, see <http://www.gnu.org/licenses/>.
+#   You should have received a copy of the GNU Lesser General Public License
+#   along with this program. If not, see <http://www.gnu.org/licenses/>.
 #
+
 from __future__ import print_function
 
-import json
+import simplejson as json
 import logging
 import os
 import sys
+from functools import partial
 
 from tornado.options import define, options
 from tornado import httpclient
@@ -38,8 +40,8 @@ AppGroup = "Application"
 CocaineGroup = "Cocaine"
 ActionGroup = "Actions"
 
-POSSIBLE_ACTIONS = ('upload', 'deploy', 'start', 'stop', 'fastdeploy',
-                    'listing', 'undeploy', 'info', 'remove')
+POSSIBLE_ACTIONS = ('upload', 'deploy', 'start', 'stop', 'restart',
+                    'fastdeploy', 'listing', 'undeploy', 'info', 'remove')
 API_VERSION = 1.3
 DEFAULT_REQUEST_TIMEOUT = 200
 
@@ -79,6 +81,15 @@ def default_format_reply(body):
     printer(body)
 
 
+def check_some_options(opts):
+    for opt in opts:
+        if getattr(options, opt, None) is None:
+            raise ValueError("Option `%s` must be set" % opt)
+
+check_version_and_appname = partial(check_some_options,
+                                    ("version", "appname"))
+
+
 def jail(f):
     def wrapper(*args, **kwargs):
         try:
@@ -171,6 +182,7 @@ def gen_task(task):
 @Dispatcher.attach("upload")
 @jail
 def upload():
+    check_some_options(("appname", "url"))
     task = {"appname": options.appname,
             "version": options.version,
             "environment": options.environment,
@@ -188,6 +200,7 @@ def upload():
 @Dispatcher.attach("start")
 @jail
 def start():
+    check_version_and_appname()
     task = {"appname": options.appname,
             "environment": options.environment,
             "version": options.version}
@@ -205,6 +218,7 @@ def start():
 @Dispatcher.attach("stop")
 @jail
 def stop():
+    check_version_and_appname()
     task = {"appname": options.appname,
             "environment": options.environment,
             "version": options.version}
@@ -217,9 +231,17 @@ def stop():
     run_request(req)
 
 
+@Dispatcher.attach("restart")
+@jail
+def restart():
+    stop()
+    start()
+
+
 @Dispatcher.attach("deploy")
 @jail
 def deploy():
+    check_some_options(("appname", "version", "profile"))
     log.info("Application %s would be deployed with profile %s."
              " Its weight would be %d",
              options.appname,
@@ -241,6 +263,7 @@ def deploy():
 @Dispatcher.attach("undeploy")
 @jail
 def undeploy():
+    check_version_and_appname()
     log.info("Application %s would be undeployed with profile.",
              options.appname)
     task = {"appname": options.appname,
@@ -258,6 +281,7 @@ def undeploy():
 @Dispatcher.attach("remove")
 @jail
 def remove():
+    check_version_and_appname()
     log.info("Application %s would be removed",
              options.appname)
     task = {"appname": options.appname,
@@ -287,7 +311,7 @@ def listing():
         try:
             apps = json.loads(body)
         except Exception:
-            log.info("%s", body)
+            print(body)
             return
 
         if len(apps) == 0:
@@ -309,11 +333,18 @@ def listing():
 @Dispatcher.attach("info")
 @jail
 def info():
+    def pretty_print(data):
+        try:
+            print(json.dumps(json.loads(data),
+                  sort_keys=True, indent=4 * ' '))
+        except Exception:
+            print(data)
+
     log.info("Fetching a list of your applications")
     task = gen_task(None)
     req = HTTPRequest(gen_url("info"),
                       method="POST",
-                      streaming_callback=default_format_reply,
+                      streaming_callback=pretty_print,
                       body=task)
     run_request(req)
 
@@ -321,6 +352,7 @@ def info():
 @Dispatcher.attach("fastdeploy")
 @jail
 def fastdeploy():
+    check_some_options(("appname", "url", "profilename"))
     task = {"appname": options.appname,
             "version": options.version,
             "environment": options.environment,
